from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .resources_get_response_results_approval_status import ResourcesGetResponse_results_approvalStatus
    from .resources_get_response_results_custom_attributes import ResourcesGetResponse_results_customAttributes
    from .resources_get_response_results_entity_type import ResourcesGetResponse_results_entityType

@dataclass
class ResourcesGetResponse_results(Parsable):
    # The approval status of the file version.For more information, see `File Status <https://help.autodesk.com/view/DOCS/ENU/?guid=Reviews_Files>`_ documentation.
    approval_status: Optional[ResourcesGetResponse_results_approvalStatus] = None
    # The time and date the file version was created.
    created_at: Optional[str] = None
    # The Autodesk ID of the user who created the version. For details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    created_by: Optional[str] = None
    # The name of the user who created the file version.
    created_by_name: Optional[str] = None
    # A list of custom attributes assigned to the file version.For more information, see `Customize Documents with Attributes <https://help.autodesk.com/view/DOCS/ENU/?guid=File_attributes>`_.
    custom_attributes: Optional[list[ResourcesGetResponse_results_customAttributes]] = None
    # The description of the file version.
    description: Optional[str] = None
    # The type of file version. Possible values:- ``SEED_FILE`` – A document that was not split into sheets when uploaded to ACC Files.- ``DOCUMENT`` – A document that was split into sheets when uploaded to ACC Files.
    entity_type: Optional[ResourcesGetResponse_results_entityType] = None
    # The file type of the version.For more details, see the `Supported Files <https://help.autodesk.com/view/BUILD/ENU/?guid=Supported_Files_Docs>`_ documentation.
    file_type: Optional[str] = None
    # The unique identifier (UUID) of the file version.
    id: Optional[UUID] = None
    # Indicates whether the file version has been deleted.``true`` – The file version is deleted, either directly or because its parent folder was deleted.``false`` – The file version is not deleted.
    is_deleted: Optional[bool] = None
    # The file name in ACC Files.Max length: 255
    name: Optional[str] = None
    # The ID (URN) of the parent folder that contains the file version.
    parent_folder_urn: Optional[str] = None
    # The URN of the file version’s storage object.
    storage_urn: Optional[str] = None
    # The time and date when the file version was last modified.
    updated_at: Optional[str] = None
    # The Autodesk ID of the user who last modified the version. For details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    updated_by: Optional[str] = None
    # The name of the user who last modified the file version.
    updated_by_name: Optional[str] = None
    # The ID (URN) of the file version in ACC.
    urn: Optional[str] = None
    # The version number of the resource in ACC Files.This number increases when the file is completely replaced (for example, re-uploaded and overwritten), not when it is merely updated or saved.
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ResourcesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ResourcesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ResourcesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .resources_get_response_results_approval_status import ResourcesGetResponse_results_approvalStatus
        from .resources_get_response_results_custom_attributes import ResourcesGetResponse_results_customAttributes
        from .resources_get_response_results_entity_type import ResourcesGetResponse_results_entityType

        from .resources_get_response_results_approval_status import ResourcesGetResponse_results_approvalStatus
        from .resources_get_response_results_custom_attributes import ResourcesGetResponse_results_customAttributes
        from .resources_get_response_results_entity_type import ResourcesGetResponse_results_entityType

        fields: dict[str, Callable[[Any], None]] = {
            "approvalStatus": lambda n : setattr(self, 'approval_status', n.get_object_value(ResourcesGetResponse_results_approvalStatus)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByName": lambda n : setattr(self, 'created_by_name', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(ResourcesGetResponse_results_customAttributes)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "entityType": lambda n : setattr(self, 'entity_type', n.get_enum_value(ResourcesGetResponse_results_entityType)),
            "fileType": lambda n : setattr(self, 'file_type', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isDeleted": lambda n : setattr(self, 'is_deleted', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "parentFolderUrn": lambda n : setattr(self, 'parent_folder_urn', n.get_str_value()),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("approvalStatus", self.approval_status)
        writer.write_str_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByName", self.created_by_name)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_str_value("description", self.description)
        writer.write_enum_value("entityType", self.entity_type)
        writer.write_str_value("fileType", self.file_type)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isDeleted", self.is_deleted)
        writer.write_str_value("name", self.name)
        writer.write_str_value("parentFolderUrn", self.parent_folder_urn)
        writer.write_str_value("storageUrn", self.storage_urn)
        writer.write_str_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("updatedByName", self.updated_by_name)
        writer.write_str_value("urn", self.urn)
        writer.write_int_value("version", self.version)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .categories_get_response import CategoriesGetResponse
    from .categories_post_request_body import CategoriesPostRequestBody
    from .categories_post_response import CategoriesPostResponse
    from .item.with_category_item_request_builder import WithCategoryItemRequestBuilder

class CategoriesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/assets/v1/projects/{projectId}/categories
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new CategoriesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/assets/v1/projects/{projectId}/categories{?filter%5BisActive%5D*,filter%5BmaxDepth%5D*,filter%5BparentId%5D*,filter%5BupdatedAt%5D*,includeUid*}", path_parameters)
    
    def by_category_id(self,category_id: str) -> WithCategoryItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.assets.v1.projects.item.categories.item collection
        param category_id: The category ID
        Returns: WithCategoryItemRequestBuilder
        """
        if category_id is None:
            raise TypeError("category_id cannot be null.")
        from .item.with_category_item_request_builder import WithCategoryItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["categoryId"] = category_id
        return WithCategoryItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[CategoriesRequestBuilderGetQueryParameters]] = None) -> Optional[CategoriesGetResponse]:
        """
        GET categories
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CategoriesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .categories_get_response import CategoriesGetResponse

        return await self.request_adapter.send_async(request_info, CategoriesGetResponse, None)
    
    async def post(self,body: CategoriesPostRequestBody, request_configuration: Optional[RequestConfiguration[CategoriesRequestBuilderPostQueryParameters]] = None) -> Optional[CategoriesPostResponse]:
        """
        POST categories
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[CategoriesPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .categories_post_response import CategoriesPostResponse

        return await self.request_adapter.send_async(request_info, CategoriesPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[CategoriesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        GET categories
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: CategoriesPostRequestBody, request_configuration: Optional[RequestConfiguration[CategoriesRequestBuilderPostQueryParameters]] = None) -> RequestInformation:
        """
        POST categories
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> CategoriesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: CategoriesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return CategoriesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class CategoriesRequestBuilderGetQueryParameters():
        """
        GET categories
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filteris_active":
                return "filter%5BisActive%5D"
            if original_name == "filtermax_depth":
                return "filter%5BmaxDepth%5D"
            if original_name == "filterparent_id":
                return "filter%5BparentId%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "include_uid":
                return "includeUid"
            return original_name
        
        # Specifies whether or not to return categories that are active. If ``true``, return only active categories. If``false``, return only inactive categories. Default is to return both active and inactive categories, to ensurefull tree structure is returned (conversely, full tree structure is not guaranteed when filtering active orinactive categories).Note that this behavior is different than that of the ``includeDeleted`` flag other endpoints use.
        filteris_active: Optional[bool] = None

        # Specifies the depth of the category tree to return, starting with the root category. Depth ``0`` means to returnonly the root category. Depth ``4``, for example, means to return only the root category and categories that arefour levels of inheritance deep, but not to return categories that are five or more levels deep.Note that this can be used in conjunction with ``filter[parentId]``, but that the depth is still computed fromthe root category, not the parent category specified by the filter.
        filtermax_depth: Optional[float] = None

        # Specifies the parent category ID of returned categories. The query returns only categories that are directchild categories of the specified category.
        filterparent_id: Optional[str] = None

        # A string that specifies a date and time or a date and time range at which all returned objects mast have beenupdated. A single date and time takes this format: ``YYYY-MM-DDThh:mm:ss.SSSZ``, A date and time range takes thisformat: ``YYYY-MM-DDThh:mm:ss.SSSZ..YYYY-MM-DDThh:mm:ss.SSSZ``. Range queries can be closed or open in eitherdirection: ``YYYY-MM-DDThh:mm:ss.SSSZ..`` or ``..YYYY-MM-DDThh:mm:ss.SSSZ``.
        filterupdated_at: Optional[str] = None

        # If provided, and set to ``true``, the globally-unique category ``uid`` field will be present in the response.The globally unique category ID is used with the (upcoming) ``v3`` category APIs. The option to include theglobally-unique ID with the ``v1`` category APIs is to help consumers transition to the new IDs.
        include_uid: Optional[bool] = None

    
    @dataclass
    class CategoriesRequestBuilderGetRequestConfiguration(RequestConfiguration[CategoriesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class CategoriesRequestBuilderPostQueryParameters():
        """
        POST categories
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "include_uid":
                return "includeUid"
            return original_name
        
        # If provided, and set to ``true``, the globally-unique category ``uid`` field will be present in the response.The globally unique category ID is used with the (upcoming) ``v3`` category APIs. The option to include theglobally-unique ID with the ``v1`` category APIs is to help consumers transition to the new IDs.
        include_uid: Optional[bool] = None

    
    @dataclass
    class CategoriesRequestBuilderPostRequestConfiguration(RequestConfiguration[CategoriesRequestBuilderPostQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class DocumentsGetResponse_status(str, Enum):
    Pending = "Pending",
    Started = "Started",
    Completed = "Completed",
    Failed = "Failed",
    PendingSign = "PendingSign",
    Signed = "Signed",




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .actions.actions_request_builder import ActionsRequestBuilder
    from .action_histories.action_histories_request_builder import ActionHistoriesRequestBuilder
    from .item.with_association_type_item_request_builder import WithAssociationTypeItemRequestBuilder

class WorkflowsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/workflows
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WorkflowsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/workflows", path_parameters)
    
    def by_association_type(self,association_type: str) -> WithAssociationTypeItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.cost.v1.containers.item.workflows.item collection
        param association_type: The type of the item with which the actions are associated. Possible values: ``FormInstance``, ``CostItem``, ``OCO``, ``PCO``, ``RCO``, ``RFQ``, ``SCO``, ``Expense``, ``Contract``, ``CostPayment``, ``BudgetPayment``, ``BudgetTransfer``, ``MainContract``.
        Returns: WithAssociationTypeItemRequestBuilder
        """
        if association_type is None:
            raise TypeError("association_type cannot be null.")
        from .item.with_association_type_item_request_builder import WithAssociationTypeItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["associationType"] = association_type
        return WithAssociationTypeItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    @property
    def action_histories(self) -> ActionHistoriesRequestBuilder:
        """
        The actionHistories property
        """
        from .action_histories.action_histories_request_builder import ActionHistoriesRequestBuilder

        return ActionHistoriesRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def actions(self) -> ActionsRequestBuilder:
        """
        The actions property
        """
        from .actions.actions_request_builder import ActionsRequestBuilder

        return ActionsRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class WithRequestGetResponse(Parsable):
    # The account ID.
    account_id: Optional[UUID] = None
    # The callback URL specified for the data request. If specified, the Data Connection service calls the URL each time a job executes for the request. The service sends a POST request that provides job execution information. The JSON payload in the POST request contains the following: ``{ ``accountId``: ``account_id``, ``requestId``: ``request_id``, ``jobId``: ``data_connector_job_id``, ``state``: ``complete``, ``success``: true or false }``.
    callback_url: Optional[str] = None
    # The date and time the data request was created, presented in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The BIM 360 / ACC user ID of the user who created the data request.
    created_by: Optional[str] = None
    # The email address of the user who created the data request.
    created_by_email: Optional[str] = None
    # The timeframe used for extracting data in the request. Currently, it is applicable only to the Activities service. This field contains the value specified in the request, indicating the range of data included in the response. Possible values:- ``TODAY``: Data for the current day (from 00:00 UTC to the time the request was made).- ``YESTERDAY``: Data for the previous calendar day (from 00:00 UTC to 23:59 UTC).- ``PAST_7_DAYS``: Data for the last 7 days, including the current day.- ``MONTH_TO_DATE``: Data from the start of the current calendar month (00:00 UTC on the 1st) to the time the request was made.- ``LAST_MONTH``: Data for the entire previous calendar month (00:00 UTC on the 1st to 23:59 UTC on the last day).
    date_range: Optional[str] = None
    # The user-entered description of this data request. If not supplied, the default value is a null string.
    description: Optional[str] = None
    # The date and time when a one-time job execution or a recurring interval schedule begins, presented in ISO 8601 format.
    effective_from: Optional[datetime.datetime] = None
    # The date and time when the recurring interval schedule ends, presented in ISO 8601 format.
    effective_to: Optional[datetime.datetime] = None
    # The end date and time for the data extraction, in ISO 8601 format.This field applies only to schemas supporting date range extraction. The detailed schema documentation delivered with each data extract identifies the schemas and tables that support date range extraction.Additional notes on using ``startDate`` and ``endDate``:- If you provide only ``startDate`` or ``endDate`` (but not both), Data Connector uses that single date for both ``startDate`` and ``endDate``.- If you request more than the Maximum Date Range Allowed for an extraction, the default date range as documented in the schema documentation is returned.- For the ``activities`` service group, data replication can be delayed up to 20 minutes, so your requests should account for that delay.
    end_date: Optional[str] = None
    # The ID of the data request.
    id: Optional[UUID] = None
    # The data request's active/inactive status. Possible values: ``true`` the request is active; ``false`` the request is inactive.
    is_active: Optional[bool] = None
    # The date and time the last job for this data request was scheduled to execute, presented in ISO 8601 format.
    last_queued_at: Optional[datetime.datetime] = None
    # (Legacy): A single project ID for the data request. Superseded by ``projectIdList``.
    project_id: Optional[UUID] = None
    # A list of up to 50 project IDs included in the data request. This field contains the project IDs for which data is being extracted. If both ``projectId`` and ``projectIdList`` were included in the request, this field contains the values from ``projectIdList``.
    project_id_list: Optional[list[str]] = None
    # The types of projects to be included in a request. The possible values are:- ``all``: - all projects (default)- ``archived``: archived projects only- ``active``: active project only
    project_status: Optional[str] = None
    # The number of ``scheduleInterval`` units to wait between job execution for the request. For example, a ``scheduleInterval`` value of ``WEEK`` and a ``reoccuringInterval`` value of ``2`` means the job will run every two weeks.
    reoccuring_interval: Optional[int] = None
    # The scheduling interval unit for jobs spawned by this data request. This value is multiplied by the ``reoccurringInterval`` attribute to specify the length of the recurring interval at which jobs run. Possible values:- ``ONE_TIME``: Run the job only once- ``DAY``: Set the recurring job interval in days- ``WEEK``: Set the recurring job interval in weeks- ``MONTH``: Set the recurring job interval in months- ``YEAR``: Set the recurring job interval in years
    schedule_interval: Optional[str] = None
    # Send a notification email to the user upon job completion. Values: true or false (default is true)
    send_email: Optional[bool] = None
    # The service groups from which data has been extracted, separated by commas.Possible values: ``all``, ``activities``, ``admin``, ``assets``, ``checklists``, ``cost``, ``dailylogs``, ``forms``, ``iq``, ``issues``, ``locations``, ``markups``, ``meetingminutes``, ``photos``, ``relationships``, ``reviews``, ``rfis``, ``schedule``, ``sheets``, ``submittals``, ``submittalsacc``, ``transmittals``.Note that the ``admin`` service includes both project and account admin, and ``all`` indicates that the extract contains all currently available service groups.
    service_groups: Optional[list[str]] = None
    # The start date and time for the data extraction, in ISO 8601 format.This field applies only to schemas supporting date range extraction. The detailed schema documentation delivered with each data extract identifies the schemas and tables that support date range extraction.Additional notes on using ``startDate`` and ``endDate``:- If you provide only ``startDate`` or ``endDate`` (but not both), Data Connector uses that single date for both ``startDate`` and ``endDate``.- If you request more than the Maximum Date Range Allowed for an extraction, the default date range as documented in the schema documentation is returned.- For the ``activities`` service group, data replication can be delayed up to 20 minutes, so your requests should account for that delay.
    start_date: Optional[str] = None
    # The date and time the data request was last updated, presented in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The BIM 360 / ACC user ID of the user who last updated the data request.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithRequestGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithRequestGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithRequestGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "accountId": lambda n : setattr(self, 'account_id', n.get_uuid_value()),
            "callbackUrl": lambda n : setattr(self, 'callback_url', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByEmail": lambda n : setattr(self, 'created_by_email', n.get_str_value()),
            "dateRange": lambda n : setattr(self, 'date_range', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "effectiveFrom": lambda n : setattr(self, 'effective_from', n.get_datetime_value()),
            "effectiveTo": lambda n : setattr(self, 'effective_to', n.get_datetime_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "lastQueuedAt": lambda n : setattr(self, 'last_queued_at', n.get_datetime_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
            "projectIdList": lambda n : setattr(self, 'project_id_list', n.get_collection_of_primitive_values(str)),
            "projectStatus": lambda n : setattr(self, 'project_status', n.get_str_value()),
            "reoccuringInterval": lambda n : setattr(self, 'reoccuring_interval', n.get_int_value()),
            "scheduleInterval": lambda n : setattr(self, 'schedule_interval', n.get_str_value()),
            "sendEmail": lambda n : setattr(self, 'send_email', n.get_bool_value()),
            "serviceGroups": lambda n : setattr(self, 'service_groups', n.get_collection_of_primitive_values(str)),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("accountId", self.account_id)
        writer.write_str_value("callbackUrl", self.callback_url)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByEmail", self.created_by_email)
        writer.write_str_value("dateRange", self.date_range)
        writer.write_str_value("description", self.description)
        writer.write_datetime_value("effectiveFrom", self.effective_from)
        writer.write_datetime_value("effectiveTo", self.effective_to)
        writer.write_str_value("endDate", self.end_date)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_datetime_value("lastQueuedAt", self.last_queued_at)
        writer.write_uuid_value("projectId", self.project_id)
        writer.write_collection_of_primitive_values("projectIdList", self.project_id_list)
        writer.write_str_value("projectStatus", self.project_status)
        writer.write_int_value("reoccuringInterval", self.reoccuring_interval)
        writer.write_str_value("scheduleInterval", self.schedule_interval)
        writer.write_bool_value("sendEmail", self.send_email)
        writer.write_collection_of_primitive_values("serviceGroups", self.service_groups)
        writer.write_str_value("startDate", self.start_date)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class PackagesGetResponse_pagination(Parsable):
    """
    Describes pagination details for the response, including information about the current page and navigation to other pages.
    """
    # The maximum number of results to be displayed on each page.
    limit: Optional[int] = None
    # The URL to retrieve the subsequent page of results, if available. If not included, this is the last page of data.
    next_url: Optional[str] = None
    # The number of results skipped before starting the current page.
    offset: Optional[int] = None
    # The URL to retrieve the preceding page of results, if applicable. Not returned on the first page of results.
    previous_url: Optional[str] = None
    # The overall count of results available across all pages.
    total_results: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PackagesGetResponse_pagination:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PackagesGetResponse_pagination
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PackagesGetResponse_pagination()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "limit": lambda n : setattr(self, 'limit', n.get_int_value()),
            "nextUrl": lambda n : setattr(self, 'next_url', n.get_str_value()),
            "offset": lambda n : setattr(self, 'offset', n.get_int_value()),
            "previousUrl": lambda n : setattr(self, 'previous_url', n.get_str_value()),
            "totalResults": lambda n : setattr(self, 'total_results', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_int_value("limit", self.limit)
        writer.write_str_value("nextUrl", self.next_url)
        writer.write_int_value("offset", self.offset)
        writer.write_str_value("previousUrl", self.previous_url)
        writer.write_int_value("totalResults", self.total_results)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .companies_get_response_results_addresses_type import CompaniesGetResponse_results_addresses_type

@dataclass
class CompaniesGetResponse_results_addresses(Parsable):
    # The street address line 1.Max length: 255
    address_line1: Optional[str] = None
    # The street address line 2.Max length: 255
    address_line2: Optional[str] = None
    # City.Max length: 255
    city: Optional[str] = None
    # Only valid country names and ISO 3166-1 alpha-2 codes will be accepted.Max length: 255
    country: Optional[str] = None
    # Phone Number.Max length: 255
    phone: Optional[str] = None
    # The zip or postal code in which this address is located.Max length: 255
    postal_code: Optional[str] = None
    # The state or province location. Only valid state/province names and ISO 3166-1 alpha-2 codes will be accepted. The provided state or province must exist in the provided country.Max length: 255
    state_or_province: Optional[str] = None
    # The address type. Will always be: ``Main``
    type: Optional[CompaniesGetResponse_results_addresses_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CompaniesGetResponse_results_addresses:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CompaniesGetResponse_results_addresses
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CompaniesGetResponse_results_addresses()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .companies_get_response_results_addresses_type import CompaniesGetResponse_results_addresses_type

        from .companies_get_response_results_addresses_type import CompaniesGetResponse_results_addresses_type

        fields: dict[str, Callable[[Any], None]] = {
            "addressLine1": lambda n : setattr(self, 'address_line1', n.get_str_value()),
            "addressLine2": lambda n : setattr(self, 'address_line2', n.get_str_value()),
            "city": lambda n : setattr(self, 'city', n.get_str_value()),
            "country": lambda n : setattr(self, 'country', n.get_str_value()),
            "phone": lambda n : setattr(self, 'phone', n.get_str_value()),
            "postalCode": lambda n : setattr(self, 'postal_code', n.get_str_value()),
            "stateOrProvince": lambda n : setattr(self, 'state_or_province', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(CompaniesGetResponse_results_addresses_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("addressLine1", self.address_line1)
        writer.write_str_value("addressLine2", self.address_line2)
        writer.write_str_value("city", self.city)
        writer.write_str_value("country", self.country)
        writer.write_str_value("phone", self.phone)
        writer.write_str_value("postalCode", self.postal_code)
        writer.write_str_value("stateOrProvince", self.state_or_province)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .get_filter_text_match_query_parameter_type import GetFilterTextMatchQueryParameterType
    from .projects_get_response import ProjectsGetResponse

class ProjectsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/admin/v1/accounts/{accountId}/users/{userId}/projects
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ProjectsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/admin/v1/accounts/{accountId}/users/{userId}/projects{?fields*,filterTextMatch*,filter%5BaccessLevels%5D*,filter%5Bclassification%5D*,filter%5Bid%5D*,filter%5BjobNumber%5D*,filter%5Bname%5D*,filter%5Bplatform%5D*,filter%5Bstatus%5D*,filter%5Btype%5D*,filter%5BupdatedAt%5D*,limit*,offset*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[ProjectsRequestBuilderGetQueryParameters]] = None) -> Optional[ProjectsGetResponse]:
        """
        Retrieves all projects for a specified user. This endpoint is compatible with both BIM 360 and Autodesk Construction Cloud (ACC) projects.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[ProjectsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .projects_get_response import ProjectsGetResponse

        return await self.request_adapter.send_async(request_info, ProjectsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[ProjectsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves all projects for a specified user. This endpoint is compatible with both BIM 360 and Autodesk Construction Cloud (ACC) projects.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> ProjectsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: ProjectsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return ProjectsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class ProjectsRequestBuilderGetQueryParameters():
        """
        Retrieves all projects for a specified user. This endpoint is compatible with both BIM 360 and Autodesk Construction Cloud (ACC) projects.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filteraccess_levels":
                return "filter%5BaccessLevels%5D"
            if original_name == "filterclassification":
                return "filter%5Bclassification%5D"
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "filterjob_number":
                return "filter%5BjobNumber%5D"
            if original_name == "filtername":
                return "filter%5Bname%5D"
            if original_name == "filterplatform":
                return "filter%5Bplatform%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "filtertype":
                return "filter%5Btype%5D"
            if original_name == "filterupdated_at":
                return "filter%5BupdatedAt%5D"
            if original_name == "filter_text_match":
                return "filterTextMatch"
            if original_name == "fields":
                return "fields"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # A comma-separated list of user project fields to include in the response. If not specified, all available fields are included by default.Possible values: ``accessLevels``, ``accountId``, ``addressLine1``, ``addressLine2``, ``city``, ``constructionType``, ``country``, ``createdAt``, ``classification``, ``deliveryMethod``, ``endDate``, ``imageUrl``, ``jobNumber``, ``latitude``, ``longitude``, ``name``, ``platform``, ``postalCode``, ``projectValue``, ``sheetCount``, ``startDate``, ``stateOrProvince``, ``status``, ``thumbnailImageUrl``, ``timezone``, ``type``, ``updatedAt``, ``contractType`` and ``currentPhase``.
        fields: Optional[list[str]] = None

        # Specifies how text-based filters should match values in supported fields.This parameter can be used in any endpoint that supports text-based filtering (e.g., ``filter[name]``, ``filter[jobNumber]``, ``filter[companyName]``, etc.).Possible values:``contains`` (default) – Matches if the field contains the specified text anywhere``startsWith`` – Matches if the field starts with the specified text``endsWith`` – Matches if the field ends with the specified text``equals`` – Matches only if the field exactly matches the specified textMatching is case-insensitive.Wildcards and regular expressions are not supported.
        filter_text_match: Optional[GetFilterTextMatchQueryParameterType] = None

        # Filters projects by user access level. Possible values: ``projectAdmin``, ``projectMember``.Max length: 255
        filteraccess_levels: Optional[list[str]] = None

        # Filters projects by classification. Possible values:``production`` – Standard production projects.``template`` – Project templates that can be cloned to create production projects.``component`` – Placeholder projects that contain standardized components (e.g., forms) for use across projects. Only one component project is permitted per account. Known as a library in the ACC unified products UI.``sample`` – The single sample project automatically created upon ACC trial setup. Only one sample project is permitted per account.Max length: 255
        filterclassification: Optional[list[str]] = None

        # A list of project IDs to filter by.
        filterid: Optional[list[str]] = None

        # Filters by a user-defined project identifier. Supports partial matches when used with ``filterTextMatch``. For example, ``filter[jobNumber]=HP-0002&filterTextMatch=equals`` returns projects where the job number is exactly ``HP-0002``.Max length: 255
        filterjob_number: Optional[str] = None

        # Filters projects by name. Supports partial matches when used with ``filterTextMatch``. For example ``filter[name]=ABCco&filterTextMatch=startsWith`` returns projects whose names start with ``ABCco``.Max length: 255
        filtername: Optional[str] = None

        # Filters by platform. Possible values: ``acc`` (Autodesk Construction Cloud) and ``bim360`` (BIM 360).Max length: 255
        filterplatform: Optional[list[str]] = None

        # Filters projects by status. Possible values: ``active``, ``pending``, ``archived``, ``suspended``.
        filterstatus: Optional[list[str]] = None

        # Filters by project type. To exclude a type, prefix it with ``-`` (e.g., ``-Bridge`` excludes bridge projects).Possible values: ``Airport``, ``Assisted Living / Nursing Home``, ``Bridge``, ``Canal / Waterway``, ``Convention Center``, ``Court House``, ``Data Center``, ``Dams / Flood Control / Reservoirs``, ``Demonstration Project``, ``Dormitory``, ``Education Facility``, ``Government Building``, ``Harbor / River Development``, ``Hospital``, ``Hotel / Motel``, ``Library``, ``Manufacturing / Factory``, ``Medical Laboratory``, ``Medical Office``, ``Military Facility``, ``Mining Facility``, ``Multi-Family Housing``, ``Museum``, ``Oil & Gas``,``Plant``, ``Office``, ``OutPatient Surgery Center``, ``Parking Structure / Garage``, ``Performing Arts``, ``Power Plant``, ``Prison / Correctional Facility``, ``Rail``, ``Recreation Building``, ``Religious Building``, ``Research Facility / Laboratory``, ``Restaurant``, ``Retail``, ``Seaport``, ``Single-Family Housing``, ``Solar Farm``, ``Stadium/Arena``, ``Streets / Roads / Highways``, ``Template Project``, ``Theme Park``, ``Training Project``, ``Transportation Building``, ``Tunnel``, ``Utilities``, ``Warehouse (non-manufacturing)``, ``Waste Water / Sewers``, ``Water Supply``, ``Wind Farm``.
        filtertype: Optional[list[str]] = None

        # Filters projects updated within a specific date range in ISO 8601 format. For example:Date range: ``2023-03-02T00:00:00.000Z..2023-03-03T23:59:59 .999Z``Specific start date: ``2023-03-02T00:00:00.000Z..``Specific end date: ``..2023-03-02T23:59:59.999Z``For more details, see `JSON API Filtering <https://jsonapi.org/format/#fetching-filtering>`_.Max length: 100
        filterupdated_at: Optional[str] = None

        # The maximum number of records to return in the response.Default: ``20``Minimum: ``1``Maximum: ``200`` (If a larger value is provided, only 200 records are returned)
        limit: Optional[int] = None

        # The index of the first record to return.Used for pagination in combination with the ``limit`` parameter.Example: ``limit=20`` and ``offset=40`` returns records 41–60.
        offset: Optional[int] = None

        # A list of fields to sort the returned user projects by. Multiple sort fields are applied in sequence order — each sort field produces groupings of projects with the same values of that field; the next sort field applies within the groupings produced by the previous sort field.Each property can be followed by a direction modifier of either ``asc`` (ascending) or ``desc`` (descending). The default is ``asc``.Possible values: ``name`` (the default), ``startDate``, ``endDate``, ``type``, ``status``, ``jobNumber``, ``constructionType``, ``deliveryMethod``, ``contractType``, ``currentPhase``, ``createdAt``, ``updatedAt`` and ``platform``.
        sort: Optional[list[str]] = None

    
    @dataclass
    class ProjectsRequestBuilderGetRequestConfiguration(RequestConfiguration[ProjectsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_relationship_get_response_entities import WithRelationshipGetResponse_entities

@dataclass
class WithRelationshipGetResponse(Parsable):
    # The date and time the relationship was created.
    created_on: Optional[datetime.datetime] = None
    # The date and time the relationship was deleted.
    deleted_on: Optional[datetime.datetime] = None
    # The entities contained in the relationship.Min items: 2 Max items: 2
    entities: Optional[list[WithRelationshipGetResponse_entities]] = None
    # The UUID that uniquely identifies the relationship.
    id: Optional[UUID] = None
    # ``true`` if this relationship is deleted.``false`` if this relationship is not deleted.
    is_deleted: Optional[bool] = None
    # ``true`` if this relationship is read only for the current caller.``false`` if this relationship is not read only for the current caller.
    is_read_only: Optional[bool] = None
    # ``true`` if this relationship was created by a service.``false`` if this relationship was not created by a service.
    is_service: Optional[bool] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithRelationshipGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithRelationshipGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithRelationshipGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_relationship_get_response_entities import WithRelationshipGetResponse_entities

        from .with_relationship_get_response_entities import WithRelationshipGetResponse_entities

        fields: dict[str, Callable[[Any], None]] = {
            "createdOn": lambda n : setattr(self, 'created_on', n.get_datetime_value()),
            "deletedOn": lambda n : setattr(self, 'deleted_on', n.get_datetime_value()),
            "entities": lambda n : setattr(self, 'entities', n.get_collection_of_object_values(WithRelationshipGetResponse_entities)),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isDeleted": lambda n : setattr(self, 'is_deleted', n.get_bool_value()),
            "isReadOnly": lambda n : setattr(self, 'is_read_only', n.get_bool_value()),
            "isService": lambda n : setattr(self, 'is_service', n.get_bool_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdOn", self.created_on)
        writer.write_datetime_value("deletedOn", self.deleted_on)
        writer.write_collection_of_object_values("entities", self.entities)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isDeleted", self.is_deleted)
        writer.write_bool_value("isReadOnly", self.is_read_only)
        writer.write_bool_value("isService", self.is_service)
    




from enum import Enum

class SubCostItemsCopyPostRequestBody_from(str, Enum):
    Contract = "contract",
    Budget = "budget",
    Estimated = "estimated",
    Proposed = "proposed",
    Submitted = "submitted",
    Approved = "approved",
    Committed = "committed",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .rfis_post_request_body_assigned_to import RfisPostRequestBody_assignedTo
    from .rfis_post_request_body_co_reviewers import RfisPostRequestBody_coReviewers
    from .rfis_post_request_body_custom_attributes import RfisPostRequestBody_customAttributes
    from .rfis_post_request_body_status import RfisPostRequestBody_status
    from .rfis_post_request_body_watchers import RfisPostRequestBody_watchers

@dataclass
class RfisPostRequestBody(Parsable):
    # The list of users assigned to the RFI.
    assigned_to: Optional[list[RfisPostRequestBody_assignedTo]] = None
    # A list of predefined categories to assign to the RFI.Categories help group RFIs for filtering and reporting. Each value must match a category configured in the project’s RFI settings. Categories are case-sensitive and project-specific.RFI categories are configured in Project Admin and may differ between projects. Call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_ to retrieve the allowed values for this field.Some possible values: ``Code Compliance``, ``Constructability``, ``Design Coordination``, ``Documentation Conflict``, ``Documentation Incomplete``, ``Field condition``, ``Other``.
    category: Optional[list[str]] = None
    # A list of reviewers assigned to the RFI. Each entry may represent a user, role, or company.
    co_reviewers: Optional[list[RfisPostRequestBody_coReviewers]] = None
    # The default cost impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.To check whether cost impact options are enabled and to retrieve the default value, call `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>`_.
    cost_impact: Optional[str] = None
    # A list of custom attributes associated with the RFI.
    custom_attributes: Optional[list[RfisPostRequestBody_customAttributes]] = None
    # The discipline associated with the RFI. To retrieve the supported values for the current project, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_.Some possible values: ``Building Management System``, ``Electrical Substation``, ``Security``, ``Audio Visual``, ``Food Service``, ``Fire Alarm``, ``Power Systems``, ``Design Systems Integrator``, ``Signage``, ``Pathways``, ``Cabling``, ``Networks``, ``Distributed Antenna System``, ``Lighting``, ``Vertical Transportation``, ``Roofing``, ``Architectural``, ``Civil/Site``, ``Concrete``, ``Electrical``, ``Exterior Envelope``, ``Fire Protection``, ``Interior/Finishes``, ``Landscaping``, ``Masonry``, ``Mechanical``, ``Plumbing``, ``Structural``, ``Other``.'
    discipline: Optional[list[str]] = None
    # The timestamp of the due date for the RFI, in the following format: YYYY-MM-DDThh:mm:ss.sz.
    due_date: Optional[str] = None
    # Unique identifier for the RFI. This is system-generated and should not be included when creating a new RFI unless the client needs to set the RFI ID in advance.
    id: Optional[UUID] = None
    # The default text for the Location field when creating a new RFI.Note that the API does not auto-populate this value.Clients are responsible for applying the default if desired.To retrieve the default value configured for this field, call `GET rfi-types </en/docs/acc/v1/reference/http/rfis-rfi-types-GET>`_.
    location_description: Optional[str] = None
    # A list of predefined location IDs associated with the RFI, based on the project’s Location Breakdown Structure (LBS). To get more information about the locations, call `GET nodes </en/docs/acc/v1/reference/http/locations-nodes-GET/>`_.
    locations: Optional[list[str]] = None
    # The default priority for new RFIs of this type.The available priority values are configured in Project Admin.If no default is set, this field is ``null``.Note that the API does not auto-populate this value when creating an RFI.Clients are responsible for applying the default if desired.The valid priority options can be retrieved by calling `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>_. Some possible values: ``null``, ``High``, ``Normal``, ``Low``.
    priority: Optional[str] = None
    # The RFI question.
    question: Optional[str] = None
    # An external ID. Typically used when the RFI was created in another system.Max length: 20
    reference: Optional[str] = None
    # The default schedule impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.To verify whether schedule impact tracking is enabled for the project and what the default value is, call `GET rfi-types <en/docs/acc/v1/reference/http/rfis-RFI-types-GET/>`_.
    schedule_impact: Optional[str] = None
    # The current status of the RFI. Available values depend on the RFI’s workflow type:- For single-reviewer workflows (US):Possible values:``draft``, ``submitted``, ``open``, ``answered``, ``rejected``, ``closed``, ``void``.- For multi-reviewer workflows (EMEA):Possible values:``draft``, ``submitted``, ``openRev1`` (manager), ``openRev2`` (reviewers), ``answeredRev1``, ``answeredManager``, ``closed``, ``void``.To determine the workflow type, call `GET users/me </en/docs/acc/v1/reference/http/rfis-users-me-GET/>`_ and check the ``workflowType`` value.For details on RFI workflows in the ACC UI, see `About RFI  Workflows – Autodesk Help <https://help.autodesk.com/view/BUILD/ENU/?guid=RFI_Workflow_Setup>`_.
    status: Optional[RfisPostRequestBody_status] = None
    # The suggested answer for the RFI.
    suggested_answer: Optional[str] = None
    # The title of the RFI.
    title: Optional[str] = None
    # A list of watchers who are notified about changes to the RFI. Each entry may represent a user, role, or company.
    watchers: Optional[list[RfisPostRequestBody_watchers]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RfisPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RfisPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RfisPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .rfis_post_request_body_assigned_to import RfisPostRequestBody_assignedTo
        from .rfis_post_request_body_co_reviewers import RfisPostRequestBody_coReviewers
        from .rfis_post_request_body_custom_attributes import RfisPostRequestBody_customAttributes
        from .rfis_post_request_body_status import RfisPostRequestBody_status
        from .rfis_post_request_body_watchers import RfisPostRequestBody_watchers

        from .rfis_post_request_body_assigned_to import RfisPostRequestBody_assignedTo
        from .rfis_post_request_body_co_reviewers import RfisPostRequestBody_coReviewers
        from .rfis_post_request_body_custom_attributes import RfisPostRequestBody_customAttributes
        from .rfis_post_request_body_status import RfisPostRequestBody_status
        from .rfis_post_request_body_watchers import RfisPostRequestBody_watchers

        fields: dict[str, Callable[[Any], None]] = {
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_collection_of_object_values(RfisPostRequestBody_assignedTo)),
            "category": lambda n : setattr(self, 'category', n.get_collection_of_primitive_values(str)),
            "coReviewers": lambda n : setattr(self, 'co_reviewers', n.get_collection_of_object_values(RfisPostRequestBody_coReviewers)),
            "costImpact": lambda n : setattr(self, 'cost_impact', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(RfisPostRequestBody_customAttributes)),
            "discipline": lambda n : setattr(self, 'discipline', n.get_collection_of_primitive_values(str)),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "locationDescription": lambda n : setattr(self, 'location_description', n.get_str_value()),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "priority": lambda n : setattr(self, 'priority', n.get_str_value()),
            "question": lambda n : setattr(self, 'question', n.get_str_value()),
            "reference": lambda n : setattr(self, 'reference', n.get_str_value()),
            "scheduleImpact": lambda n : setattr(self, 'schedule_impact', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(RfisPostRequestBody_status)),
            "suggestedAnswer": lambda n : setattr(self, 'suggested_answer', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(RfisPostRequestBody_watchers)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("assignedTo", self.assigned_to)
        writer.write_collection_of_primitive_values("category", self.category)
        writer.write_collection_of_object_values("coReviewers", self.co_reviewers)
        writer.write_str_value("costImpact", self.cost_impact)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_collection_of_primitive_values("discipline", self.discipline)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("locationDescription", self.location_description)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("priority", self.priority)
        writer.write_str_value("question", self.question)
        writer.write_str_value("reference", self.reference)
        writer.write_str_value("scheduleImpact", self.schedule_impact)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("suggestedAnswer", self.suggested_answer)
        writer.write_str_value("title", self.title)
        writer.write_collection_of_object_values("watchers", self.watchers)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .workflow_get_response_project_roles_mapping import WorkflowGetResponse_projectRolesMapping
    from .workflow_get_response_workflow_type import WorkflowGetResponse_workflowType

@dataclass
class WorkflowGetResponse(Parsable):
    # A description of the workflow
    description: Optional[str] = None
    # A list of project roles and their corresponding permitted assignees.
    project_roles_mapping: Optional[list[WorkflowGetResponse_projectRolesMapping]] = None
    # The region type of the workflow. Possible values: ``US``, ``EU``.
    workflow_type: Optional[WorkflowGetResponse_workflowType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .workflow_get_response_project_roles_mapping import WorkflowGetResponse_projectRolesMapping
        from .workflow_get_response_workflow_type import WorkflowGetResponse_workflowType

        from .workflow_get_response_project_roles_mapping import WorkflowGetResponse_projectRolesMapping
        from .workflow_get_response_workflow_type import WorkflowGetResponse_workflowType

        fields: dict[str, Callable[[Any], None]] = {
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "projectRolesMapping": lambda n : setattr(self, 'project_roles_mapping', n.get_collection_of_object_values(WorkflowGetResponse_projectRolesMapping)),
            "workflowType": lambda n : setattr(self, 'workflow_type', n.get_enum_value(WorkflowGetResponse_workflowType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("description", self.description)
        writer.write_collection_of_object_values("projectRolesMapping", self.project_roles_mapping)
        writer.write_enum_value("workflowType", self.workflow_type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .relationships_intersect_post_request_body_entities import RelationshipsIntersectPostRequestBody_entities
    from .relationships_intersect_post_request_body_with_entities import RelationshipsIntersectPostRequestBody_withEntities

@dataclass
class RelationshipsIntersectPostRequestBody(Parsable):
    # The list of entities to return relationships for.Min items: 1 Max items: 20
    entities: Optional[list[RelationshipsIntersectPostRequestBody_entities]] = None
    # The optional list of entities to filter returned relationships by.Min items: 1 Max items: 20
    with_entities: Optional[list[RelationshipsIntersectPostRequestBody_withEntities]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RelationshipsIntersectPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RelationshipsIntersectPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RelationshipsIntersectPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .relationships_intersect_post_request_body_entities import RelationshipsIntersectPostRequestBody_entities
        from .relationships_intersect_post_request_body_with_entities import RelationshipsIntersectPostRequestBody_withEntities

        from .relationships_intersect_post_request_body_entities import RelationshipsIntersectPostRequestBody_entities
        from .relationships_intersect_post_request_body_with_entities import RelationshipsIntersectPostRequestBody_withEntities

        fields: dict[str, Callable[[Any], None]] = {
            "entities": lambda n : setattr(self, 'entities', n.get_collection_of_object_values(RelationshipsIntersectPostRequestBody_entities)),
            "withEntities": lambda n : setattr(self, 'with_entities', n.get_collection_of_object_values(RelationshipsIntersectPostRequestBody_withEntities)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("entities", self.entities)
        writer.write_collection_of_object_values("withEntities", self.with_entities)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class FormsGetResponse_data_tabularValues_materialsEntries(Parsable):
    # Indicates if the material log row has been deleted.
    deleted: Optional[bool] = None
    # Additional description of the materials.
    description: Optional[str] = None
    # Unique identifier for the material log row.
    id: Optional[str] = None
    # Text indicating the material used.
    item: Optional[str] = None
    # The quantity of material that was used.
    quantity: Optional[float] = None
    # The unit of measure for the quantity specified.
    unit: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> FormsGetResponse_data_tabularValues_materialsEntries:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: FormsGetResponse_data_tabularValues_materialsEntries
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return FormsGetResponse_data_tabularValues_materialsEntries()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "deleted": lambda n : setattr(self, 'deleted', n.get_bool_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "item": lambda n : setattr(self, 'item', n.get_str_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("deleted", self.deleted)
        writer.write_str_value("description", self.description)
        writer.write_str_value("id", self.id)
        writer.write_str_value("item", self.item)
        writer.write_float_value("quantity", self.quantity)
        writer.write_str_value("unit", self.unit)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class StatusStepSetsPostResponse_values(Parsable):
    # An immutable string assigned to a status on its creation, typically created by incorporating thestatus's label into a larger string that is guaranteed to be unique within a project. The bucket nameremains the same for the life of the status, and doesn't change even if the label does. The bucket valueis a useful tool for semantic identification.
    bucket: Optional[str] = None
    # The color of the status as the status appears in the Assets UI.This field is not restricted as it is primarily a tool for the Assets UI to use.However, there are only certain colors that the Autodesk Construction Cloud Assets UI knows how tooperate with. Understood color values are: ``adsk-black``, ``adsk-white``, ``adsk-charcoal-900``,``adsk-charcoal-800``, ``adsk-charcoal-700``, ``adsk-charcoal-600``, ``adsk-charcoal-500``,``adsk-charcoal-400``, ``adsk-charcoal-300``, ``adsk-charcoal-200``, ``adsk-charcoal-100``,``adsk-charcoal-050``, ``adsk-blue-700``, ``adsk-blue-500``, ``adsk-blue-300``, ``adsk-blue-100``,``adsk-red-700``, ``adsk-red-500``, ``adsk-red-300``, ``adsk-green-700``, ``adsk-green-500``,``adsk-green-300``, ``adsk-yellow-orange-700``, ``adsk-yellow-orange-500``, ``adsk-yellow-orange-300``,``adsk-dark-blue-700``, ``adsk-dark-blue-500``, ``adsk-dark-blue-300``, ``adsk-pink-700``,``adsk-pink-500``, ``adsk-pink-300``, ``adsk-turquoise-700``, ``adsk-turquoise-500``,``adsk-turquoise-300``, ``adsk-purple-700``, ``adsk-purple-500``, ``adsk-purple-300``,``adsk-salmon-700``, ``adsk-salmon-500``, ``adsk-salmon-300``, ``adsk-brown-700``, ``adsk-brown-500``,``adsk-brown-300``.Using colors other than those specified here is not disallowed, but may result in unexpected behavior inthe Assets UI.
    color: Optional[str] = None
    # The time when the component was created (ISO8601 Date time format in UTC).
    created_at: Optional[str] = None
    # The actor that created the component. This is an Autodesk / Oxygen ID.
    created_by: Optional[str] = None
    # The time when the component was deleted at (ISO8601 Date time format in UTC).
    deleted_at: Optional[str] = None
    # The actor that deleted the component. This is an Autodesk / Oxygen ID.
    deleted_by: Optional[str] = None
    # A description of the status.
    description: Optional[str] = None
    # The ID of the component.
    id: Optional[str] = None
    # A flag indicating whether the component is active or inactive (``isActive`` is ``true`` if-and-only-if ``deletedAt`` is empty).
    is_active: Optional[bool] = None
    # The display label of the status. This label appears in the Assets UI when viewing statuses. The label mustbe unique within this status set, but does not need to be unique within the project (in other words, thesame label can be used in multiple status sets in the same project). ``Uniqueness`` in this context meanscase-insensitive.
    label: Optional[str] = None
    # The Autodesk Construction Cloud project ID. Must be a UUID or a project ID of the form ``b.{UUID}``.
    project_id: Optional[UUID] = None
    # A value that indicates the order of a status within its status set. Each status in the set has a sortorder value that indicates its order relative to other statuses in the set. A status set's sort ordervalues don't necessarily start at 1, and may not be sequential. The only way to know a status's orderwithin a set is to compare its sort order value with the sort order values of other statuses.
    sort_order: Optional[int] = None
    # The ID of the status set to which the new status belongs.
    status_step_set_id: Optional[UUID] = None
    # The time when the component was last updated (ISO8601 Date time format in UTC).
    updated_at: Optional[str] = None
    # The actor that last updated the component. This is an Autodesk / Oxygen ID.
    updated_by: Optional[str] = None
    # A global sequence number that is incremented any time a component of this type is created, updated, or deleted. If you cache components, you can use the version value to compare the cached component to the same component online to see if the component has been updated. If the online component has a higher version value, it has been updated.
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatusStepSetsPostResponse_values:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatusStepSetsPostResponse_values
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatusStepSetsPostResponse_values()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "bucket": lambda n : setattr(self, 'bucket', n.get_str_value()),
            "color": lambda n : setattr(self, 'color', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_str_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_str_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "label": lambda n : setattr(self, 'label', n.get_str_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_uuid_value()),
            "sortOrder": lambda n : setattr(self, 'sort_order', n.get_int_value()),
            "statusStepSetId": lambda n : setattr(self, 'status_step_set_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("bucket", self.bucket)
        writer.write_str_value("color", self.color)
        writer.write_str_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("description", self.description)
        writer.write_str_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_str_value("label", self.label)
        writer.write_uuid_value("projectId", self.project_id)
        writer.write_int_value("sortOrder", self.sort_order)
        writer.write_uuid_value("statusStepSetId", self.status_step_set_id)
        writer.write_str_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_int_value("version", self.version)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class VersionsBatchGetPostRequestBody(Parsable):
    # A list of version IDs or item IDs. If you use item IDs it retrieves the values for the latest (tip) versions. You can specify up to 50 documents. To find the version ID and item ID of a document follow the initial steps of the `Download Files </en/docs/bim360/v1/tutorials/document-management/download-document-s3/>`_ tutorial.
    urns: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> VersionsBatchGetPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: VersionsBatchGetPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return VersionsBatchGetPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "urns": lambda n : setattr(self, 'urns', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("urns", self.urns)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .attributes_post_request_body_possible_values import AttributesPostRequestBody_possibleValues
    from .attributes_post_request_body_status import AttributesPostRequestBody_status
    from .attributes_post_request_body_type import AttributesPostRequestBody_type

@dataclass
class AttributesPostRequestBody(Parsable):
    # The description of the attribute as shown in the UI.Max length: 1000
    description: Optional[str] = None
    # ``true``: users can select more than one value for this attribute.``false``: (default) users can select only one value.
    multiple_choice: Optional[bool] = None
    # The name of the custom attribute as displayed in the UI.Max length: 50
    name: Optional[str] = None
    # A list of possible values for the attribute.
    possible_values: Optional[list[AttributesPostRequestBody_possibleValues]] = None
    # The display status of the attribute in the UI. Possible values: ``active``, ``inactive``, ``hidden``.
    status: Optional[AttributesPostRequestBody_status] = None
    # The type of the attribute.Possible values: ``text``, ``numeric``
    type: Optional[AttributesPostRequestBody_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AttributesPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AttributesPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AttributesPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .attributes_post_request_body_possible_values import AttributesPostRequestBody_possibleValues
        from .attributes_post_request_body_status import AttributesPostRequestBody_status
        from .attributes_post_request_body_type import AttributesPostRequestBody_type

        from .attributes_post_request_body_possible_values import AttributesPostRequestBody_possibleValues
        from .attributes_post_request_body_status import AttributesPostRequestBody_status
        from .attributes_post_request_body_type import AttributesPostRequestBody_type

        fields: dict[str, Callable[[Any], None]] = {
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "multipleChoice": lambda n : setattr(self, 'multiple_choice', n.get_bool_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "possibleValues": lambda n : setattr(self, 'possible_values', n.get_collection_of_object_values(AttributesPostRequestBody_possibleValues)),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(AttributesPostRequestBody_status)),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(AttributesPostRequestBody_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("description", self.description)
        writer.write_bool_value("multipleChoice", self.multiple_choice)
        writer.write_str_value("name", self.name)
        writer.write_collection_of_object_values("possibleValues", self.possible_values)
        writer.write_enum_value("status", self.status)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .cost_items_get_response_results_scope import CostItemsGetResponse_results_scope
    from .cost_items_get_response_results_split_number import CostItemsGetResponse_results_splitNumber

@dataclass
class CostItemsGetResponse_results(Parsable):
    # The amount approved by the owner.
    approved: Optional[float] = None
    # The ID of the budget to which the cost item is linked.
    budget_id: Optional[str] = None
    # The current budget status of the cost item, indicating its lifecycle stage.Possible values: ``draft``, ``open``, ``submitted``, ``accepted``, ``approved``, ``executed``, ``revising``, ``rejected``, ``void``.Note that you can only change the status by updating the associated change order.
    budget_status: Optional[str] = None
    # Not relevant
    budget_transfer_id: Optional[str] = None
    # Not relevant
    budget_transfer_item_id: Optional[str] = None
    # The amount committed to the supplier.
    committed: Optional[float] = None
    # The exchange rate used for the ``committed`` cost. If multi-currency was not enabled, the value is set to ``1`` regardless of the specified rate. Default value: ``1``.
    committed_exchange_rate: Optional[float] = None
    # The ID of the contract to which the cost item is linked.
    contract_id: Optional[str] = None
    # The current financial approval status of the cost item.Possible values: ``draft``, ``open``, ``revising``, ``pricing``, ``proposed``, ``approved``, ``executed``, ``rejected``, ``contracted``, ``accepted``, ``void``.Note that the cost status can only be changed by updating the associated change order.
    cost_status: Optional[str] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # A detailed description of the cost item.
    description: Optional[str] = None
    # The rough estimate of the cost item without a formal quote.
    estimated: Optional[float] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The unique identifier of the cost item.
    id: Optional[UUID] = None
    # The initial quantity of the proposed cost item as specified in the budget or contract.
    input_quantity: Optional[float] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time that the item's integration status was last changed.
    integration_state_changed_at: Optional[str] = None
    # The user who last changed the integration status. This is the ID of a user managed by the BIM 360/ACC Admin.
    integration_state_changed_by: Optional[str] = None
    # Indicates if the cost item includes a markup.``true``: the item is a markup cost item.``false``: (default) the item is not a markup cost item.
    is_markup: Optional[bool] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # A list of the IDs of the project locations where this item applies, along with the node paths of these locations in the project's locations tree.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    location_paths: Optional[list[str]] = None
    # A list of the IDs of the project locations where this item applies.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    locations: Optional[list[str]] = None
    # The name of the cost item.Max length: 255
    name: Optional[str] = None
    # A note associated with the cost item, formatted as Tiptap rich text (https://tiptap.dev/introduction/).
    note: Optional[str] = None
    # The system generated sequential number.
    number: Optional[str] = None
    # The quoted cost of the cost item provided by the supplier.
    proposed: Optional[float] = None
    # The exchange rate used for the ``proposed`` cost. If multi-currency was not enabled, the value is set to ``1`` regardless of the specified rate. Default value: ``1``.
    proposed_exchange_rate: Optional[float] = None
    # The total quantity of the cost item.
    quantity: Optional[float] = None
    # Defines the project's scope status of the cost item. Possible values: ``out``, ``in``, ``tbd``, ``budgetOnly``, ``contingency``.
    scope: Optional[CostItemsGetResponse_results_scope] = None
    # The scope of work of the cost items, formatted as Tiptap rich text (https://tiptap.dev/introduction/).
    scope_of_work: Optional[str] = None
    # Not relevant
    split_number: Optional[CostItemsGetResponse_results_splitNumber] = None
    # The amount submitted to the owner for approval.
    submitted: Optional[float] = None
    # The category or classification of the cost item, which the Project Admin can customize.
    type: Optional[str] = None
    # The unit of measurement for the cost item.
    unit: Optional[str] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CostItemsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CostItemsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CostItemsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .cost_items_get_response_results_scope import CostItemsGetResponse_results_scope
        from .cost_items_get_response_results_split_number import CostItemsGetResponse_results_splitNumber

        from .cost_items_get_response_results_scope import CostItemsGetResponse_results_scope
        from .cost_items_get_response_results_split_number import CostItemsGetResponse_results_splitNumber

        fields: dict[str, Callable[[Any], None]] = {
            "approved": lambda n : setattr(self, 'approved', n.get_float_value()),
            "budgetId": lambda n : setattr(self, 'budget_id', n.get_str_value()),
            "budgetStatus": lambda n : setattr(self, 'budget_status', n.get_str_value()),
            "budgetTransferId": lambda n : setattr(self, 'budget_transfer_id', n.get_str_value()),
            "budgetTransferItemId": lambda n : setattr(self, 'budget_transfer_item_id', n.get_str_value()),
            "committed": lambda n : setattr(self, 'committed', n.get_float_value()),
            "committedExchangeRate": lambda n : setattr(self, 'committed_exchange_rate', n.get_float_value()),
            "contractId": lambda n : setattr(self, 'contract_id', n.get_str_value()),
            "costStatus": lambda n : setattr(self, 'cost_status', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "estimated": lambda n : setattr(self, 'estimated', n.get_float_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "integrationStateChangedAt": lambda n : setattr(self, 'integration_state_changed_at', n.get_str_value()),
            "integrationStateChangedBy": lambda n : setattr(self, 'integration_state_changed_by', n.get_str_value()),
            "isMarkup": lambda n : setattr(self, 'is_markup', n.get_bool_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "locationPaths": lambda n : setattr(self, 'location_paths', n.get_collection_of_primitive_values(str)),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "note": lambda n : setattr(self, 'note', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "proposed": lambda n : setattr(self, 'proposed', n.get_float_value()),
            "proposedExchangeRate": lambda n : setattr(self, 'proposed_exchange_rate', n.get_float_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "scope": lambda n : setattr(self, 'scope', n.get_enum_value(CostItemsGetResponse_results_scope)),
            "scopeOfWork": lambda n : setattr(self, 'scope_of_work', n.get_str_value()),
            "splitNumber": lambda n : setattr(self, 'split_number', n.get_object_value(CostItemsGetResponse_results_splitNumber)),
            "submitted": lambda n : setattr(self, 'submitted', n.get_float_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("approved", self.approved)
        writer.write_str_value("budgetId", self.budget_id)
        writer.write_str_value("budgetStatus", self.budget_status)
        writer.write_str_value("budgetTransferId", self.budget_transfer_id)
        writer.write_str_value("budgetTransferItemId", self.budget_transfer_item_id)
        writer.write_float_value("committed", self.committed)
        writer.write_float_value("committedExchangeRate", self.committed_exchange_rate)
        writer.write_str_value("contractId", self.contract_id)
        writer.write_str_value("costStatus", self.cost_status)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("description", self.description)
        writer.write_float_value("estimated", self.estimated)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_uuid_value("id", self.id)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_str_value("integrationStateChangedAt", self.integration_state_changed_at)
        writer.write_str_value("integrationStateChangedBy", self.integration_state_changed_by)
        writer.write_bool_value("isMarkup", self.is_markup)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_collection_of_primitive_values("locationPaths", self.location_paths)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("name", self.name)
        writer.write_str_value("note", self.note)
        writer.write_str_value("number", self.number)
        writer.write_float_value("proposed", self.proposed)
        writer.write_float_value("proposedExchangeRate", self.proposed_exchange_rate)
        writer.write_float_value("quantity", self.quantity)
        writer.write_enum_value("scope", self.scope)
        writer.write_str_value("scopeOfWork", self.scope_of_work)
        writer.write_object_value("splitNumber", self.split_number)
        writer.write_float_value("submitted", self.submitted)
        writer.write_str_value("type", self.type)
        writer.write_str_value("unit", self.unit)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_workflow_get_response_steps_candidates import WithWorkflowGetResponse_steps_candidates
    from .with_workflow_get_response_steps_due_date_type import WithWorkflowGetResponse_steps_dueDateType
    from .with_workflow_get_response_steps_group_review import WithWorkflowGetResponse_steps_groupReview
    from .with_workflow_get_response_steps_type import WithWorkflowGetResponse_steps_type

@dataclass
class WithWorkflowGetResponse_steps(Parsable):
    # (Displayed in the UI when selecting reviewers for a step)Lists the users, roles, or companies that were configured as reviewers for this step.These candidates are defined during workflow setup and determine who will be invited to participate in the step during a review.
    candidates: Optional[WithWorkflowGetResponse_steps_candidates] = None
    # Specifies how the due date is calculated for this step. It works together with duration.This field applies only to ``REVIEWER`` and ``APPROVER`` steps.Possible values:- ``CALENDAR_DAY`` (default): the due date includes all calendar days, including weekends and holidays.- ``WORKDAY``: the due date excludes weekends and project holidays.
    due_date_type: Optional[WithWorkflowGetResponse_steps_dueDateType] = None
    # (``Time allowed`` in the UI) The number of days allocated to complete this step.This field applies only to ``REVIEWER`` and ``APPROVER`` steps. It is used to calculate the due date based on the selected ``dueDateType``.Valid range: ``1–99``.
    duration: Optional[int] = None
    # (``Reviewer Type`` in the UI) Defines whether multiple reviewers can participate in this step and how their responses are handled. It applies only to ``REVIEWER`` steps.
    group_review: Optional[WithWorkflowGetResponse_steps_groupReview] = None
    # The ID of the step.
    id: Optional[str] = None
    # The name of the step, as defined in the workflow. It appears in the UI and is used in workflow configuration. Maximum length: 255 characters.Max length: 255
    name: Optional[str] = None
    # Indicates the step type in the workflow. Possible values:- ``INITIATOR``: the first step. It typically represents the person who launches the review.- ``REVIEWER``: an intermediate step. It allows one or more reviewers to evaluate the files.- ``APPROVER``: the final step. It represents the decision maker who approves or rejects the files.
    type: Optional[WithWorkflowGetResponse_steps_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithWorkflowGetResponse_steps:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithWorkflowGetResponse_steps
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithWorkflowGetResponse_steps()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_workflow_get_response_steps_candidates import WithWorkflowGetResponse_steps_candidates
        from .with_workflow_get_response_steps_due_date_type import WithWorkflowGetResponse_steps_dueDateType
        from .with_workflow_get_response_steps_group_review import WithWorkflowGetResponse_steps_groupReview
        from .with_workflow_get_response_steps_type import WithWorkflowGetResponse_steps_type

        from .with_workflow_get_response_steps_candidates import WithWorkflowGetResponse_steps_candidates
        from .with_workflow_get_response_steps_due_date_type import WithWorkflowGetResponse_steps_dueDateType
        from .with_workflow_get_response_steps_group_review import WithWorkflowGetResponse_steps_groupReview
        from .with_workflow_get_response_steps_type import WithWorkflowGetResponse_steps_type

        fields: dict[str, Callable[[Any], None]] = {
            "candidates": lambda n : setattr(self, 'candidates', n.get_object_value(WithWorkflowGetResponse_steps_candidates)),
            "dueDateType": lambda n : setattr(self, 'due_date_type', n.get_enum_value(WithWorkflowGetResponse_steps_dueDateType)),
            "duration": lambda n : setattr(self, 'duration', n.get_int_value()),
            "groupReview": lambda n : setattr(self, 'group_review', n.get_object_value(WithWorkflowGetResponse_steps_groupReview)),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(WithWorkflowGetResponse_steps_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("candidates", self.candidates)
        writer.write_enum_value("dueDateType", self.due_date_type)
        writer.write_int_value("duration", self.duration)
        writer.write_object_value("groupReview", self.group_review)
        writer.write_str_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_sub_cost_items_patch_response_type import WithSubCostItemsPatchResponse_type

@dataclass
class WithSubCostItemsPatchResponse(Parsable):
    # The identifier for the sub cost item. If copied from another model, for example, a ``Contract SOV``, it inherits the code. Otherwise, it is manually assigned.Max length: 255
    code: Optional[str] = None
    # The ID of the cost item to which the sub cost item belongs.
    cost_item_id: Optional[str] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The unique ID of the sub cost item.
    id: Optional[UUID] = None
    # The recorded input quantity, typically used in performance tracking. For example, in labor tracking, ``inputQuantity`` represents man-hours utilized.
    input_quantity: Optional[float] = None
    # The name of the sub cost item.Max length: 1024
    name: Optional[str] = None
    # The unique ID of the parent sub cost item. A sub cost item has a parent Id when it is part of a hierarchical cost structure. Root sub cost items do not have a parent ID.
    parent_id: Optional[str] = None
    # The position of the sub cost item relative to its sibling sub cost items. If a new sub cost item is assigned a position that already exists, the system shifts existing items downward to maintain order.
    position: Optional[float] = None
    # The planned number of units allocated for the sub cost item.
    quantity: Optional[float] = None
    # The classification of the sub cost item that indicates its role in cost tracking and approval workflows. Possible values: ``estimated``, ``proposed``, ``submitted``, ``approved``, ``committed``.Max length: 255
    type: Optional[WithSubCostItemsPatchResponse_type] = None
    # The unit of measurement for the sub cost item. This value is configured in the ``Unit of measure`` settings for the project. Common units include ``ea`` (Each), ``gal`` (Gallon), and various volume, length, and time measurements.Max length: 1024
    unit: Optional[str] = None
    # The price per individual unit of the sub cost item.
    unit_price: Optional[float] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The total value of the sub cost item, calculated as ``quantity`` * ``unitPrice``.
    value: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithSubCostItemsPatchResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithSubCostItemsPatchResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithSubCostItemsPatchResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_sub_cost_items_patch_response_type import WithSubCostItemsPatchResponse_type

        from .with_sub_cost_items_patch_response_type import WithSubCostItemsPatchResponse_type

        fields: dict[str, Callable[[Any], None]] = {
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "costItemId": lambda n : setattr(self, 'cost_item_id', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "parentId": lambda n : setattr(self, 'parent_id', n.get_str_value()),
            "position": lambda n : setattr(self, 'position', n.get_float_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(WithSubCostItemsPatchResponse_type)),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "value": lambda n : setattr(self, 'value', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("code", self.code)
        writer.write_str_value("costItemId", self.cost_item_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_uuid_value("id", self.id)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_str_value("name", self.name)
        writer.write_str_value("parentId", self.parent_id)
        writer.write_float_value("position", self.position)
        writer.write_float_value("quantity", self.quantity)
        writer.write_enum_value("type", self.type)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_float_value("value", self.value)
    




from enum import Enum

class IssueAttributeMappingsGetResponse_results_mappedItemType(str, Enum):
    IssueType = "issueType",
    IssueCategory = "Issue Category",
    IssueSubtype = "issueSubtype",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .issues_post_request_body_custom_attributes_value import IssuesPostRequestBody_customAttributes_value

@dataclass
class IssuesPostRequestBody_customAttributes(Parsable):
    # The unique identifier of the custom attribute.
    attribute_definition_id: Optional[UUID] = None
    # Custom attribute value. Possible value types: ``string``, ``number``, ``null``.
    value: Optional[IssuesPostRequestBody_customAttributes_value] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssuesPostRequestBody_customAttributes:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssuesPostRequestBody_customAttributes
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssuesPostRequestBody_customAttributes()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .issues_post_request_body_custom_attributes_value import IssuesPostRequestBody_customAttributes_value

        from .issues_post_request_body_custom_attributes_value import IssuesPostRequestBody_customAttributes_value

        fields: dict[str, Callable[[Any], None]] = {
            "attributeDefinitionId": lambda n : setattr(self, 'attribute_definition_id', n.get_uuid_value()),
            "value": lambda n : setattr(self, 'value', n.get_object_value(IssuesPostRequestBody_customAttributes_value)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("attributeDefinitionId", self.attribute_definition_id)
        writer.write_object_value("value", self.value)
    




from enum import Enum

class ViewsGetResponse_modelSetViewVersions_documentVersions_documentStatus(str, Enum):
    Succeeded = "Succeeded",
    Failed = "Failed",
    Running = "Running",
    Skipped = "Skipped",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class SheetsGetResponse_results_viewable(Parsable):
    """
    Information about the sheet relevant for loading the sheet to the Viewer. See `Add Viewer to an HTML Page <https://forge.autodesk.com/en/docs/viewer/v7/developers_guide/viewer_basics/>`_ for more information.
    """
    # The GUID of the viewable resources.When loading the sheet to the viewer, this GUID should be used to find the related geometry node of the sheet.
    guid: Optional[str] = None
    # The URN of the viewable resources. Multiple sheets created by the same original file may share the same viewable URN.When loading the sheet to the viewer, this URN should be used as the documentId to get the manifest.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SheetsGetResponse_results_viewable:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SheetsGetResponse_results_viewable
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SheetsGetResponse_results_viewable()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "guid": lambda n : setattr(self, 'guid', n.get_str_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("guid", self.guid)
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_export_get_response_result_output_failed_files import WithExportGetResponse_result_output_failedFiles

@dataclass
class WithExportGetResponse_result_output(Parsable):
    """
    The output containing the downloadable signed URL.
    """
    # Only for dwg/rvt files.
    failed_files: Optional[list[WithExportGetResponse_result_output_failedFiles]] = None
    # The signed URL to download the PDF or ZIP file. Expires in 1 hour.
    signed_url: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithExportGetResponse_result_output:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithExportGetResponse_result_output
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithExportGetResponse_result_output()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_export_get_response_result_output_failed_files import WithExportGetResponse_result_output_failedFiles

        from .with_export_get_response_result_output_failed_files import WithExportGetResponse_result_output_failedFiles

        fields: dict[str, Callable[[Any], None]] = {
            "failedFiles": lambda n : setattr(self, 'failed_files', n.get_collection_of_object_values(WithExportGetResponse_result_output_failedFiles)),
            "signedUrl": lambda n : setattr(self, 'signed_url', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("failedFiles", self.failed_files)
        writer.write_str_value("signedUrl", self.signed_url)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class TimeSheetsGetResponse_results(Parsable):
    # The BIM 360/ACC ID of the user who made the last change to the timesheet.
    changed_by: Optional[str] = None
    # The ID of the cost container for the project to which the timesheet belongs.
    container_id: Optional[UUID] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The BIM 360/ACC ID of the user who created the timesheet.
    creator_id: Optional[str] = None
    # The last date of the time period covered by the timesheet. This is also the date that the tracked ``inputQuantity`` and ``outputQuantity`` values are considered to have been reported.
    end_date: Optional[str] = None
    # The ID of the timesheet.
    id: Optional[UUID] = None
    # The hours worked during the time period covered by the timesheet.
    input_quantity: Optional[float] = None
    # The input unit of measurement of the timesheet. Currently this value should always be ``hr``.
    input_unit: Optional[str] = None
    # The quantity of material used during time period covered by the timesheet.
    output_quantity: Optional[float] = None
    # The output unit of measurement of the timesheet.
    output_unit: Optional[str] = None
    # The first date of the time period covered by the timesheet.
    start_date: Optional[str] = None
    # The ID of the tracking item instance to which the timesheet belongs.
    tracking_item_instance_id: Optional[UUID] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TimeSheetsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TimeSheetsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TimeSheetsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "changedBy": lambda n : setattr(self, 'changed_by', n.get_str_value()),
            "containerId": lambda n : setattr(self, 'container_id', n.get_uuid_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "creatorId": lambda n : setattr(self, 'creator_id', n.get_str_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "inputUnit": lambda n : setattr(self, 'input_unit', n.get_str_value()),
            "outputQuantity": lambda n : setattr(self, 'output_quantity', n.get_float_value()),
            "outputUnit": lambda n : setattr(self, 'output_unit', n.get_str_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "trackingItemInstanceId": lambda n : setattr(self, 'tracking_item_instance_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("changedBy", self.changed_by)
        writer.write_uuid_value("containerId", self.container_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("creatorId", self.creator_id)
        writer.write_str_value("endDate", self.end_date)
        writer.write_uuid_value("id", self.id)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_str_value("inputUnit", self.input_unit)
        writer.write_float_value("outputQuantity", self.output_quantity)
        writer.write_str_value("outputUnit", self.output_unit)
        writer.write_str_value("startDate", self.start_date)
        writer.write_uuid_value("trackingItemInstanceId", self.tracking_item_instance_id)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_export_get_response_result_error import WithExportGetResponse_result_error
    from .with_export_get_response_result_output import WithExportGetResponse_result_output

@dataclass
class WithExportGetResponse_result(Parsable):
    """
    The result of a completed export job.- If the ``status`` is ``successful``, a downloadable signed URL will be included in the ``result.output`` object.- If the ``status`` value is ``failed`` (e.g., because some files were deleted), the ``result.error`` object will include details of the error.
    """
    # Information about the error.
    error: Optional[WithExportGetResponse_result_error] = None
    # Details about the downloadable signed URL.
    output: Optional[WithExportGetResponse_result_output] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithExportGetResponse_result:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithExportGetResponse_result
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithExportGetResponse_result()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_export_get_response_result_error import WithExportGetResponse_result_error
        from .with_export_get_response_result_output import WithExportGetResponse_result_output

        from .with_export_get_response_result_error import WithExportGetResponse_result_error
        from .with_export_get_response_result_output import WithExportGetResponse_result_output

        fields: dict[str, Callable[[Any], None]] = {
            "error": lambda n : setattr(self, 'error', n.get_object_value(WithExportGetResponse_result_error)),
            "output": lambda n : setattr(self, 'output', n.get_object_value(WithExportGetResponse_result_output)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("error", self.error)
        writer.write_object_value("output", self.output)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .item.with_package_item_request_builder import WithPackageItemRequestBuilder
    from .packages_get_response import PackagesGetResponse
    from .packages_post_request_body import PackagesPostRequestBody
    from .packages_post_response import PackagesPostResponse

class PackagesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/takeoff/v1/projects/{projectId}/packages
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new PackagesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/takeoff/v1/projects/{projectId}/packages{?limit*,offset*}", path_parameters)
    
    def by_package_id(self,package_id: UUID) -> WithPackageItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.takeoff.v1.projects.item.packages.item collection
        param package_id: The takeoff package ID.To find the ID, call `GET packages </en/docs/acc/v1/reference/http/takeoff-projects-project_id-packages-GET/>`_.
        Returns: WithPackageItemRequestBuilder
        """
        if package_id is None:
            raise TypeError("package_id cannot be null.")
        from .item.with_package_item_request_builder import WithPackageItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["packageId"] = package_id
        return WithPackageItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[PackagesRequestBuilderGetQueryParameters]] = None) -> Optional[PackagesGetResponse]:
        """
        Retrieves the takeoff packages for a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PackagesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .packages_get_response import PackagesGetResponse

        return await self.request_adapter.send_async(request_info, PackagesGetResponse, None)
    
    async def post(self,body: PackagesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[PackagesPostResponse]:
        """
        Creates a takeoff package for a project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PackagesPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .packages_post_response import PackagesPostResponse

        return await self.request_adapter.send_async(request_info, PackagesPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[PackagesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves the takeoff packages for a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: PackagesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a takeoff package for a project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> PackagesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: PackagesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return PackagesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class PackagesRequestBuilderGetQueryParameters():
        """
        Retrieves the takeoff packages for a project.
        """
        # The maximum number of packages per page.Acceptable values: ``1-200``.Default value: ``200``.
        limit: Optional[int] = None

        # The package index from which the pagination starts. This is zero-based.
        offset: Optional[int] = None

    
    @dataclass
    class PackagesRequestBuilderGetRequestConfiguration(RequestConfiguration[PackagesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class PackagesRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class ChangeOrdersGetResponse(Parsable):
    # The status and rules of the change order.
    act_as: Optional[str] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # Unique auto-generated identifier of the change order type.
    id: Optional[UUID] = None
    # The name of the change order type.
    name: Optional[str] = None
    # Not relevant
    position: Optional[int] = None
    # The type of the change order type.
    type: Optional[str] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    # Not relevant
    workflow_type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ChangeOrdersGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ChangeOrdersGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ChangeOrdersGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "actAs": lambda n : setattr(self, 'act_as', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "position": lambda n : setattr(self, 'position', n.get_int_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "workflowType": lambda n : setattr(self, 'workflow_type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("actAs", self.act_as)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_int_value("position", self.position)
        writer.write_str_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("workflowType", self.workflow_type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .item.with_node_item_request_builder import WithNodeItemRequestBuilder
    from .nodes_get_response import NodesGetResponse
    from .nodes_post_request_body import NodesPostRequestBody
    from .nodes_post_response import NodesPostResponse
    from .post_insert_option_query_parameter_type import PostInsertOptionQueryParameterType

class NodesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/locations/v2/projects/{projectId}/trees/{treeId}/nodes
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new NodesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/locations/v2/projects/{projectId}/trees/{treeId}/nodes{?filter%5Bid%5D*,insertOption*,limit*,offset*,targetNodeId*}", path_parameters)
    
    def by_node_id(self,node_id: UUID) -> WithNodeItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.locations.v2.projects.item.trees.item.nodes.item collection
        param node_id: The unique identifier of an LBS node.To find node IDs, call the `GET nodes </en/docs/acc/v1/reference/http/locations-nodes-GET/>`_ endpoint and check the value of ``results.id`` in the returned nodes.
        Returns: WithNodeItemRequestBuilder
        """
        if node_id is None:
            raise TypeError("node_id cannot be null.")
        from .item.with_node_item_request_builder import WithNodeItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["nodeId"] = node_id
        return WithNodeItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[NodesRequestBuilderGetQueryParameters]] = None) -> Optional[NodesGetResponse]:
        """
        Retrieves an array of nodes (locations) from the specified locations tree (LBS). Returns all nodes in the tree by default.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[NodesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .nodes_get_response import NodesGetResponse

        return await self.request_adapter.send_async(request_info, NodesGetResponse, None)
    
    async def post(self,body: NodesPostRequestBody, request_configuration: Optional[RequestConfiguration[NodesRequestBuilderPostQueryParameters]] = None) -> Optional[NodesPostResponse]:
        """
        Creates a node in the specified locations tree. Note that creating the root node is not allowed because the root node is created automatically when the project is created.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[NodesPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .nodes_post_response import NodesPostResponse

        return await self.request_adapter.send_async(request_info, NodesPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[NodesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves an array of nodes (locations) from the specified locations tree (LBS). Returns all nodes in the tree by default.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: NodesPostRequestBody, request_configuration: Optional[RequestConfiguration[NodesRequestBuilderPostQueryParameters]] = None) -> RequestInformation:
        """
        Creates a node in the specified locations tree. Note that creating the root node is not allowed because the root node is created automatically when the project is created.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> NodesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: NodesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return NodesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class NodesRequestBuilderGetQueryParameters():
        """
        Retrieves an array of nodes (locations) from the specified locations tree (LBS). Returns all nodes in the tree by default.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # Specifies one or more nodes (locations) in the LBS tree to retrieve. Separate multiple node IDs with commas (no space); for example, ``filter[id]=88e07ccb-4594-4dc5-8973-304412b8fa96,de9aca33-5e0c-4668-85fa-f96273db4b35``.To find node IDs, call this endpoint and check the value of ``results.id`` in the returned nodes.Note that when you use this parameter, the server ignores the ``limit`` and ``offset`` parameters, and each node in the response includes a ``path`` array containing its ancestor nodes in the tree.
        filterid: Optional[list[str]] = None

        # The maximum number of location nodes to return per page. Acceptable values: ``1-10000``. Default value: ``10000``.
        limit: Optional[int] = None

        # The node index at which the pagination starts. This is zero-based; for example, with a value of ``6``, the response starts with the seventh node.
        offset: Optional[int] = None

    
    @dataclass
    class NodesRequestBuilderGetRequestConfiguration(RequestConfiguration[NodesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class NodesRequestBuilderPostQueryParameters():
        """
        Creates a node in the specified locations tree. Note that creating the root node is not allowed because the root node is created automatically when the project is created.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "insert_option":
                return "insertOption"
            if original_name == "target_node_id":
                return "targetNodeId"
            return original_name
        
        # Where to insert the new node relative to the target node that you specified with ``targetNodeId``. The nodes in a given tier of the tree have a defined sequence order, and ``insertOption`` specifies whether the new node comes before or after the target node.Required only when ``targetNodeId`` is also included in the request.Possible values: ``After``, ``Before``
        insert_option: Optional[PostInsertOptionQueryParameterType] = None

        # Unique identifier of a node that will be the new node's immediate sibling in the locations tree. The target node's ``parentId`` must match the ``body.parentId`` field in the request.Note that nodes in a given tier of the tree have a defined sequence order. The new node will be created in the same tier, either before or after the target node. You specify the sequence position of the new node using the  ``insertOption`` parameter.Note that this parameter is unavailable if this request is creating an existing node's first child node.Required only when ``insertOption`` is also included in the request.
        target_node_id: Optional[UUID] = None

    
    @dataclass
    class NodesRequestBuilderPostRequestConfiguration(RequestConfiguration[NodesRequestBuilderPostQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_version_item_request_builder import WithVersionItemRequestBuilder

class VersionsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/reviews/v1/projects/{projectId}/versions
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new VersionsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/reviews/v1/projects/{projectId}/versions", path_parameters)
    
    def by_version_id(self,version_id: str) -> WithVersionItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.reviews.v1.projects.item.versions.item collection
        param version_id: The URL-encoded unique identifier (URN) of the file version whose review and approval history you want to retrieve.For example, encode ``urn:adsk.wipprod:fs.file:vf.Ibsc4cPuQEqBHRJdBjhr6w?version=2``as ``urn%3Aadsk.wipprod%3Afs.file%3Avf.Ibsc4cPuQEqBHRJdBjhr6w%3Fversion%3D2``.To find the latest version, call `GET versions </en/docs/acc/v1/reference/http/reviews-getreviewversions-GET/>`_ and check the ``urn`` field.
        Returns: WithVersionItemRequestBuilder
        """
        if version_id is None:
            raise TypeError("version_id cannot be null.")
        from .item.with_version_item_request_builder import WithVersionItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["versionId"] = version_id
        return WithVersionItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_form_patch_response_form_template_status import WithFormPatchResponse_formTemplate_status

@dataclass
class WithFormPatchResponse_formTemplate(Parsable):
    """
    Information about the form's template.
    """
    # The unique identifier of the template.
    id: Optional[str] = None
    # Name of the form template.
    name: Optional[str] = None
    # Unique indentifier of the project the template belongs to.
    project_id: Optional[str] = None
    # Possible values: ``active``, ``inactive``, ``deleted``
    status: Optional[WithFormPatchResponse_formTemplate_status] = None
    # User defined type of the form template.
    template_type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithFormPatchResponse_formTemplate:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithFormPatchResponse_formTemplate
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithFormPatchResponse_formTemplate()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_form_patch_response_form_template_status import WithFormPatchResponse_formTemplate_status

        from .with_form_patch_response_form_template_status import WithFormPatchResponse_formTemplate_status

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "projectId": lambda n : setattr(self, 'project_id', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithFormPatchResponse_formTemplate_status)),
            "templateType": lambda n : setattr(self, 'template_type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_str_value("projectId", self.project_id)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("templateType", self.template_type)
    

